@@ -1,4 +1,4 @@
-/*  Copyright (C) 2005-2011, Axis Communications AB, LUND, SWEDEN
+/*  Copyright (C) 2005-2016, Axis Communications AB, LUND, SWEDEN
  *
  *  This file is part of RAPP.
  *
@@ -37,7 +37,6 @@
 #include <math.h>
 #include <signal.h>
 #include <unistd.h>
-#include <dlfcn.h>
 #include <sys/time.h>
 #include "rapp.h"
 
@@ -114,7 +113,8 @@ typedef struct rapp_bmark_data_st {
     uint8_t *pad;     /**< Source buffer, set and padded with zeros  */
     uint8_t *clear;   /**< Source buffer with all bits cleared       */
     uint8_t *checker; /**< Source buffer with checker bit pattern    */
-    uint8_t *aux;     /**< Auxillary buffer, for LUTs etc            */
+    uint8_t *aux;     /**< Auxiliary buffer, for LUTs etc            */
+    uint8_t *aux2;    /**< Second similar auxiliary buffer */
     uint8_t *src[5];  /**< A table of pointers to the 5 src buffers  */
     int      dim_u8;  /**< 8-bit image buffer dimension              */
     int      dim_bin; /**< Binary image buffer dimension             */
@@ -193,6 +193,9 @@ rapp_bmark_exec_u8_u8(int (*func)(), const int *args);
 static void
 rapp_bmark_exec_u8_u8_p(int (*func)(), const int *args);
 
+static void
+rapp_bmark_exec_thresh_pixel(int (*func)(), const int *args);
+
 static void
 rapp_bmark_exec_expand(int (*func)(), const int *args);
 
@@ -208,6 +211,9 @@ rapp_bmark_exec_rotate_u8(int (*func)(), const int *args);
 static void
 rapp_bmark_exec_cond_set_u8(int (*func)(), const int *args);
 
+static void
+rapp_bmark_exec_cond_u8_u8(int (*func)(), const int *args);
+
 static void
 rapp_bmark_exec_cond_copy_u8(int (*func)(), const int *args);
 
@@ -273,6 +279,10 @@ static const rapp_bmark_table_t rapp_bmark_suite[] = {
     RAPP_BMARK_ENTRY(thresh_lt_u8,   NULL, u8_bin, 7, 0),
     RAPP_BMARK_ENTRY(thresh_gtlt_u8, NULL, u8_bin, 7, 9),
     RAPP_BMARK_ENTRY(thresh_ltgt_u8, NULL, u8_bin, 7, 9),
+    RAPP_BMARK_ENTRY(thresh_gt_pixel_u8,   NULL, thresh_pixel, 1, 0),
+    RAPP_BMARK_ENTRY(thresh_lt_pixel_u8,   NULL, thresh_pixel, 1, 0),
+    RAPP_BMARK_ENTRY(thresh_gtlt_pixel_u8, NULL, thresh_pixel, 2, 0),
+    RAPP_BMARK_ENTRY(thresh_ltgt_pixel_u8, NULL, thresh_pixel, 2, 0),
     /* rapp_reduce functions */
     RAPP_BMARK_ENTRY(reduce_1x2_u8, NULL, u8_u8, 0, 0),
     RAPP_BMARK_ENTRY(reduce_2x1_u8, NULL, u8_u8, 0, 0),
@@ -363,9 +373,15 @@ static const rapp_bmark_table_t rapp_bmark_suite[] = {
     RAPP_BMARK_ENTRY(cond_set_u8,  "empty",   cond_set_u8,  2,  0),
     RAPP_BMARK_ENTRY(cond_set_u8,  "full",    cond_set_u8,  0,  0),
     RAPP_BMARK_ENTRY(cond_set_u8,  "checker", cond_set_u8,  3,  0),
+    RAPP_BMARK_ENTRY(cond_addc_u8, "empty",   cond_set_u8,  2,  1),
+    RAPP_BMARK_ENTRY(cond_addc_u8, "full",    cond_set_u8,  0,  1),
+    RAPP_BMARK_ENTRY(cond_addc_u8, "checker", cond_set_u8,  3,  1),
     RAPP_BMARK_ENTRY(cond_copy_u8, "empty",   cond_copy_u8, 2,  0),
     RAPP_BMARK_ENTRY(cond_copy_u8, "full",    cond_copy_u8, 0,  0),
     RAPP_BMARK_ENTRY(cond_copy_u8, "checker", cond_copy_u8, 3,  0),
+    RAPP_BMARK_ENTRY(cond_add_u8,  "empty",   cond_u8_u8,   2,  0),
+    RAPP_BMARK_ENTRY(cond_add_u8,  "full",    cond_u8_u8,   0,  0),
+    RAPP_BMARK_ENTRY(cond_add_u8,  "checker", cond_u8_u8,   3,  0),
     /* rapp_gather functions */
     RAPP_BMARK_ENTRY(gather_u8, "empty, 1 row",    gather_u8, 2,  1),
     RAPP_BMARK_ENTRY(gather_u8, "full, 1 row",     gather_u8, 0,  1),
@@ -555,6 +571,7 @@ rapp_bmark_setup(int width, int height)
     rapp_bmark_data.clear   = rapp_malloc(size, 0);
     rapp_bmark_data.checker = rapp_malloc(size, 0);
     rapp_bmark_data.aux     = rapp_malloc(size, 0);
+    rapp_bmark_data.aux2    = rapp_malloc(size, 0);
 
     rapp_bmark_data.dim_u8  = dim_u8;
     rapp_bmark_data.dim_bin = dim_bin;
@@ -572,6 +589,7 @@ rapp_bmark_setup(int width, int height)
     memset(rapp_bmark_data.clear,   0,    size);
     memset(rapp_bmark_data.checker, 0x55, size);
     memset(rapp_bmark_data.aux,     0xff, size);
+    memset(rapp_bmark_data.aux2,    0xff, size);
 
     rapp_bmark_data.dst     += offset;
     rapp_bmark_data.set     += offset;
@@ -579,6 +597,7 @@ rapp_bmark_setup(int width, int height)
     rapp_bmark_data.clear   += offset;
     rapp_bmark_data.checker += offset;
     rapp_bmark_data.aux     += offset;
+    rapp_bmark_data.aux2    += offset;
 
     rapp_bmark_data.src[0] = rapp_bmark_data.set;
     rapp_bmark_data.src[1] = rapp_bmark_data.pad;
@@ -600,6 +619,7 @@ rapp_bmark_cleanup(void)
     rapp_free(&rapp_bmark_data.clear  [-rapp_bmark_data.offset]);
     rapp_free(&rapp_bmark_data.checker[-rapp_bmark_data.offset]);
     rapp_free(&rapp_bmark_data.aux    [-rapp_bmark_data.offset]);
+    rapp_free(&rapp_bmark_data.aux2   [-rapp_bmark_data.offset]);
 }
 
 static void
@@ -753,6 +773,27 @@ rapp_bmark_exec_u8_u8_p(int (*func)(), const int *args)
             data->aux);
 }
 
+static void
+rapp_bmark_exec_thresh_pixel(int (*func)(), const int *args)
+{
+    const rapp_bmark_data_t *data = &rapp_bmark_data;
+
+    const int num_thresholds = args[0];
+    if (num_thresholds == 2) {
+        (*func)(data->dst, data->dim_bin,
+                data->set, data->dim_u8,
+                data->aux, data->dim_u8,
+                data->aux2, data->dim_u8,
+                data->width, data->height);
+    }
+    else {
+        (*func)(data->dst, data->dim_bin,
+                data->set, data->dim_u8,
+                data->aux, data->dim_u8,
+                data->width, data->height);
+    }
+}
+
 static void
 rapp_bmark_exec_expand(int (*func)(), const int *args)
 {
@@ -799,7 +840,18 @@ rapp_bmark_exec_cond_set_u8(int (*func)(), const int *args)
     int                      idx  = args[0];
     (*func)(data->dst, data->dim_u8,
             data->src[idx], data->dim_bin,
-            data->width, data->height, 0);
+            data->width, data->height, args[1]);
+}
+
+static void
+rapp_bmark_exec_cond_u8_u8(int (*func)(), const int *args)
+{
+    const rapp_bmark_data_t *data = &rapp_bmark_data;
+    int                      idx  = args[0];
+    (*func)(data->dst, data->dim_u8,
+            data->set, data->dim_u8,
+            data->src[idx], data->dim_bin,
+            data->width, data->height, args[1]);
 }
 
 static void
