@@ -1,4 +1,4 @@
-/*  Copyright (C) 2005-2011, Axis Communications AB, LUND, SWEDEN
+/*  Copyright (C) 2005-2016, Axis Communications AB, LUND, SWEDEN
  *
  *  This file is part of RAPP.
  *
@@ -109,6 +109,9 @@
  *    - rc_vec_vadmx.h: Axis VADMX.
  *         Uses 64-bit VADMX registers.
  *         Implements the complete interface.
+ *    - rc_vec_vadmx2.h: Axis VADMX2.
+ *         Uses 128-bit VADMX2 registers.
+ *         Implements the complete interface.
  *    - rc_vec_loongson2ef.h: Loongson-2E and -2F.
  *         Uses 64-bit vector registers.
  *         Implements the complete interface.
@@ -661,7 +664,7 @@ typedef arch_vector_t rc_vec_t;
 /**
  *  Linear interpolation.
  *  Computes dstv = srcv1 + ((blend8*(srcv2 - srcv1) + 0x80) >> 8) for each
- *  8-bit field. The Q.8 blend factor @e blend8 must be in the range [0,0x7f].
+ *  8-bit field. The Q.8 blend factor @e blend8 must be in the range [0x1,0x7f].
  *
  *  @param dstv    The output vector.
  *  @param srcv1   The first input vector.
@@ -683,7 +686,7 @@ typedef arch_vector_t rc_vec_t;
  *  Linear interpolation rounded towards srcv1.
  *  Computes dstv = srcv1 + (blend8*(srcv2 - srcv1)/256) for each 8-bit
  *  field, with the update term rounded towards zero. The Q.8 blend factor
- *  @e blend8 must be in the range [0,0x7f].
+ *  @e blend8 must be in the range [0x1,0x7f].
  *
  *  @param dstv    The output vector.
  *  @param srcv1   The first input vector.
@@ -705,7 +708,7 @@ typedef arch_vector_t rc_vec_t;
  *  Linear interpolation rounded towards srcv2.
  *  Computes dstv = srcv1 + (blend8*(srcv2 - srcv1)/256) for each 8-bit
  *  field, with the update term rounded away from zero. The Q.8 blend factor
- *  @e blend8 must be in the range [0,0x7f].
+ *  @e blend8 must be in the range [0x1,0x7f].
  *
  *  @param dstv    The output vector.
  *  @param srcv1   The first input vector.
@@ -751,6 +754,19 @@ typedef arch_vector_t rc_vec_t;
  */
 #define RC_VEC_GETMASKV(maskv, vec)
 
+/**
+ *  Set 8-bit vector fields from a binary mask vector.
+ *  Unpack each of the left-most bits from the binary mask vector into
+ *  the corresponding 8-bit field in the output vector, as the value
+ *  zero or 0xff, as many as fits into the output vector. Except for
+ *  producing the specific value 0xff for bits with the value one, this
+ *  is the inversion of RC_VEC_GETMASKV.
+ *
+ *  @param  vec    The output vector.
+ *  @param  maskv  The input mask vector.
+ */
+#define RC_VEC_SETMASKV(vec, maskv)
+
 /* @} */
 
 
