@@ -1,4 +1,4 @@
-/*  Copyright (C) 2005-2010, Axis Communications AB, LUND, SWEDEN
+/*  Copyright (C) 2005-2012, Axis Communications AB, LUND, SWEDEN
  *
  *  This file is part of RAPP.
  *
@@ -40,47 +40,16 @@
 
 #include <emmintrin.h>  /* SSE2 intrinsics */
 
+/* See the porting documentation for generic comments. */
 
-/*
- * -------------------------------------------------------------
- *  Performance hints
- * -------------------------------------------------------------
- */
-
-/**
- *  Use CMPGE instead of CMPGT when possible.
- */
 #define RC_VEC_HINT_CMPGE
 
-/**
- *  Use AVGR/SUBHR instead of AVGT/SUBHT when possible.
- */
 #define RC_VEC_HINT_AVGR
 
-
-/*
- * -------------------------------------------------------------
- *  The vector type
- * -------------------------------------------------------------
- */
-
-/**
- *  The vector type definition.
- */
 typedef __m128i rc_vec_t;
 
-/**
- *  The number of bytes in a vector.
- */
 #define RC_VEC_SIZE 16
 
-
-/*
- * -------------------------------------------------------------
- *  Vector state
- * -------------------------------------------------------------
- */
-
 /**
  *  No global declarations is needed for SSE2 registers.
  */
@@ -91,35 +60,12 @@ typedef __m128i rc_vec_t;
  */
 #define RC_VEC_CLEANUP()
 
-
-/*
- * -------------------------------------------------------------
- *  Memory access
- * -------------------------------------------------------------
- */
-
-/**
- *  Load a vector from memory.
- */
 #define RC_VEC_LOAD(vec, ptr) \
     ((vec) = _mm_load_si128((const rc_vec_t*)(ptr)))
 
-/**
- *  Store a vector in memory.
- */
 #define RC_VEC_STORE(ptr, vec) \
     _mm_store_si128((rc_vec_t*)(ptr), vec)
 
-
-/*
- * -------------------------------------------------------------
- *  Misaligned memory access
- * -------------------------------------------------------------
- */
-
-/**
- *  Initialize a sequence of misaligned loads.
- */
 #define RC_VEC_LDINIT(vec1, vec2, vec3, uptr, ptr) \
 do {                                               \
     (void)(vec1);                                  \
@@ -128,37 +74,16 @@ do {                                               \
     (uptr) = (ptr);                                \
 } while (0)
 
-/**
- *  Misaligned vector load from memory.
- *  Uses the adjusted data pointer uptr.
- */
+/* We only need to use the adjusted data pointer uptr. */
 #define RC_VEC_LOADU(dstv, vec1, vec2, vec3, uptr) \
     ((dstv) = _mm_loadu_si128((const rc_vec_t*)(uptr)))
 
-
-/*
- * -------------------------------------------------------------
- *  Field relocation
- * -------------------------------------------------------------
- */
-
-/**
- *  Shift all fields to the logical left by a constant value.
- */
 #define RC_VEC_SHLC(dstv, srcv, bytes) \
     ((dstv) = _mm_srli_si128(srcv, bytes))
 
-/**
- *  Shift fields to the logical right by a constant value.
- */
 #define RC_VEC_SHRC(dstv, srcv, bytes) \
     ((dstv) = _mm_slli_si128(srcv, bytes))
 
-/**
- *  Align srcv1 and srcv2 to dstv, starting at field @e bytes
- *  into concatenation of srcv1 and srcv2. The alignment
- *  value @e bytes must be a constant.
- */
 #define RC_VEC_ALIGNC(dstv, srcv1, srcv2, bytes)                   \
 do {                                                               \
     rc_vec_t sv1__ = _mm_srli_si128(srcv1, bytes);                 \
@@ -166,11 +91,6 @@ do {                                                               \
     (dstv) = _mm_or_si128(sv1__, sv2__);                           \
 } while (0)
 
-/**
- *  Pack the even fields of srcv1 and srcv2 into one vector,
- *  with fields 0, ..., RC_VEC_SIZE/2 - 1 from srcv1 and
- *  fields RC_VEC_SIZE/2, ..., RC_VEC_SIZE - 1 from srcv2.
- */
 #define RC_VEC_PACK(dstv, srcv1, srcv2)           \
 do {                                              \
     rc_vec_t sel__ = _mm_set1_epi16(0x00ff);      \
@@ -179,52 +99,24 @@ do {                                              \
     (dstv) = _mm_packus_epi16(sv1__, sv2__);      \
 } while (0)
 
-
-/*
- * -------------------------------------------------------------
- *  Bitwise logical operations
- * -------------------------------------------------------------
- */
-
-/**
- *  Set all bits to zero.
- */
 #define RC_VEC_ZERO(vec) \
     ((vec) = _mm_setzero_si128())
 
-/**
- *  Bitwise NOT.
- */
 #define RC_VEC_NOT(dstv, srcv) \
     ((dstv) = _mm_andnot_si128(srcv, _mm_set1_epi8(0xff)))
 
-/**
- *  Bitwise AND.
- */
 #define RC_VEC_AND(dstv, srcv1, srcv2) \
     ((dstv) = _mm_and_si128(srcv1, srcv2))
 
-/**
- *  Bitwise OR.
- */
 #define RC_VEC_OR(dstv, srcv1, srcv2) \
     ((dstv) = _mm_or_si128(srcv1, srcv2))
 
-/**
- *  Bitwise XOR.
- */
 #define RC_VEC_XOR(dstv, srcv1, srcv2) \
     ((dstv) = _mm_xor_si128(srcv1, srcv2))
 
-/**
- *  Bitwise AND NOT.
- */
 #define RC_VEC_ANDNOT(dstv, srcv1, srcv2) \
     ((dstv) = _mm_andnot_si128(srcv2, srcv1))
 
-/**
- *  Bitwise OR NOT.
- */
 #define RC_VEC_ORNOT(dstv, srcv1, srcv2) \
 do {                                     \
     rc_vec_t inv__;                      \
@@ -232,9 +124,6 @@ do {                                     \
     RC_VEC_OR(dstv, srcv1, inv__);       \
 } while (0)
 
-/**
- *  Bitwise XOR NOT.
- */
 #define RC_VEC_XORNOT(dstv, srcv1, srcv2) \
 do {                                      \
     rc_vec_t inv__;                       \
@@ -242,25 +131,9 @@ do {                                      \
     RC_VEC_XOR(dstv, srcv1, inv__);       \
 } while (0)
 
-
-/*
- * -------------------------------------------------------------
- *  Arithmetic operations on 8-bit fields
- * -------------------------------------------------------------
- */
-
-/**
- *  Set a value to all fields.
- *  Sets the scalar value 'scal' in each 8-bit field.
- */
 #define RC_VEC_SPLAT(vec, scal) \
     ((vec) = _mm_set1_epi8(scal))
 
-/**
- *  Absolute value.
- *  Computes dstv = 2*abs(srcv - 0x80) for each 8-bit field.
- *  The result is saturated to [0,0xff].
- */
 #define RC_VEC_ABS(dstv, srcv)                     \
 do {                                               \
     rc_vec_t sv__   = (srcv);                      \
@@ -271,17 +144,9 @@ do {                                               \
     (dstv) = _mm_adds_epu8(pos__, pos__);          \
 } while (0)
 
-/**
- *  Saturating addition.
- *  Computes dstv = MIN(srcv1 + srcv2, 0xff) for each 8-bit field.
- */
 #define RC_VEC_ADDS(dstv, srcv1, srcv2) \
     ((dstv) = _mm_adds_epu8(srcv1, srcv2))
 
-/**
- *  Average value, truncated.
- *  Computes dstv = (srcv1 + srcv2) >> 1 for each 8-bit field.
- */
 #define RC_VEC_AVGT(dstv, srcv1, srcv2)                 \
 do {                                                    \
     rc_vec_t sv1__ = (srcv1);                           \
@@ -293,54 +158,25 @@ do {                                                    \
     (dstv) = _mm_sub_epi8(sv1__, adj__);                \
 } while (0)
 
-/**
- *  Average value, rounded.
- *  Computes dstv = (srcv1 + srcv2 + 1) >> 1 for each 8-bit field.
- */
 #define RC_VEC_AVGR(dstv, srcv1, srcv2) \
     ((dstv) = _mm_avg_epu8(srcv1, srcv2))
 
-/**
- *  Average value, rounded towards srcv1.
- *  Computes dstv = (srcv1 + srcv2 + (srcv1 > srcv2)) >> 1
- *  for each 8-bit field.
- */
 #define RC_VEC_AVGZ(dstv, srcv1, srcv2) \
     RC_VEC_LERPZ(dstv, srcv1, srcv2, 0x80, _mm_set1_epi16(0x8000))
 
-/**
- *  Saturating subtraction.
- *  Computes dstv = MAX(srcv1 - srcv2, 0) for each 8-bit field.
- */
 #define RC_VEC_SUBS(dstv, srcv1, srcv2) \
     ((dstv) = _mm_subs_epu8(srcv1, srcv2))
 
-/**
- *  Absolute-value subtraction.
- *  Computes dstv = ABS(srcv1 - srcv2) for each 8-bit field.
- */
 #define RC_VEC_SUBA(dstv, srcv1, srcv2)                 \
     ((dstv) = _mm_or_si128(_mm_subs_epu8(srcv1, srcv2), \
                            _mm_subs_epu8(srcv2, srcv1)))
 
-/**
- *  Half subtraction with bias, truncated.
- *  Computes dstv = (srcv1 - srcv2 + 0xff) >> 1 for each 8-bit field.
- */
 #define RC_VEC_SUBHT(dstv, srcv1, srcv2) \
     RC_VEC_AVGT(dstv, srcv1, _mm_sub_epi8(_mm_set1_epi8(0xff), srcv2))
 
-/**
- *  Half subtraction with bias, rounded.
- *  Computes dstv = (srcv1 - srcv2 + 0x100) >> 1 for each 8-bit field.
- */
 #define RC_VEC_SUBHR(dstv, srcv1, srcv2) \
     RC_VEC_AVGR(dstv, srcv1, _mm_sub_epi8(_mm_set1_epi8(0xff), srcv2))
 
-/**
- *  Comparison.
- *  Computes dstv = srcv1 > srcv2 ? 0xff : 0 for each 8-bit field.
- */
 #define RC_VEC_CMPGT(dstv, srcv1, srcv2)                                \
 do {                                                                    \
     rc_vec_t sv1__ = (srcv1);                                           \
@@ -348,41 +184,21 @@ do {                                                                    \
     (dstv) = _mm_andnot_si128(lte__, _mm_set1_epi8(0xff));              \
 } while (0)
 
-/**
- *  Comparison.
- *  Computes dstv = srcv1 >= srcv2 ? 0xff : 0 for each 8-bit field.
- */
 #define RC_VEC_CMPGE(dstv, srcv1, srcv2)                      \
 do {                                                          \
     rc_vec_t sv__ = (srcv1);                                  \
     (dstv) = _mm_cmpeq_epi8(sv__, _mm_max_epu8(sv__, srcv2)); \
 } while (0)
 
-/**
- *  Minimum value.
- *  Computes dstv = MIN(srcv1, srcv2) for each 8-bit field.
- */
 #define RC_VEC_MIN(dstv, srcv1, srcv2) \
     ((dstv) = _mm_min_epu8(srcv1, srcv2))
 
-/**
- *  Maximum value.
- *  Computes dstv = MAX(srcv1, srcv2) for each 8-bit field.
- */
 #define RC_VEC_MAX(dstv, srcv1, srcv2) \
     ((dstv) = _mm_max_epu8(srcv1, srcv2))
 
-/**
- *  Generate the blend vector needed by RC_VEC_LERP().
- */
 #define RC_VEC_BLEND(blendv, blend8) \
     ((blendv) = _mm_set1_epi16((blend8) << 8))
 
-/**
- *  Linear interpolation.
- *  Computes dstv = srcv1 + ((blend8*(srcv2 - srcv1) + 0x80) >> 8) for each
- *  8-bit field. The Q.8 blend factor @e blend8 must be in the range [0,0x7f].
- */
 #define RC_VEC_LERP(dstv, srcv1, srcv2, blend8, blendv)                     \
 do {                                                                        \
     rc_vec_t srcv1__ = (srcv1);                                             \
@@ -393,18 +209,9 @@ do {                                                                        \
     RC_VEC_LERP__(dstv, srcv1__, srcv2__, blend__, bias__, bias__, zero__); \
 } while (0)
 
-/**
- *  Generate the blend vector needed by RC_VEC_LERPZ().
- */
 #define RC_VEC_BLENDZ(blendv, blend8) \
     ((blendv) = _mm_set1_epi16((blend8) << 8))
 
-/**
- *  Linear interpolation rounded towards srcv1.
- *  Computes dstv = srcv1 + (blend8*(srcv2 - srcv1)/256) for each 8-bit
- *  field, with the update term rounded towards zero. The Q.8 blend factor
- *  @e blend8 must be in the range [0,0x7f].
- */
 #define RC_VEC_LERPZ(dstv, srcv1, srcv2, blend8, blendv)                  \
 do {                                                                      \
     rc_vec_t srcv1__ = (srcv1);                                           \
@@ -418,18 +225,9 @@ do {                                                                      \
     RC_VEC_LERP__(dstv, srcv1__, srcv2__, blend__, blo__, bhi__, zero__); \
 } while (0)
 
-/**
- *  Generate the blend vector needed by RC_VEC_LERPN().
- */
 #define RC_VEC_BLENDN(blendv, blend8) \
     RC_VEC_BLENDZ(blendv, blend8)
 
-/**
- *  Linear interpolation rounded towards srcv2.
- *  Computes dstv = srcv1 + (blend8*(srcv2 - srcv1)/256) for each 8-bit
- *  field, with the update term rounded away from zero. The Q.8 blend factor
- *  @e blend8 must be in the range [0,0x7f].
- */
 #define RC_VEC_LERPN(dstv, srcv1, srcv2, blend8, blendv)                  \
 do {                                                                      \
     rc_vec_t srcv1__ = (srcv1);                                           \
@@ -443,40 +241,26 @@ do {                                                                      \
     RC_VEC_LERP__(dstv, srcv1__, srcv2__, blend__, blo__, bhi__, zero__); \
 } while (0)
 
-
-/*
- * -------------------------------------------------------------
- *  Binary mask operations
- * -------------------------------------------------------------
- */
-
-/**
- *  Pack the most significant bits in each 8-bit field to the
- *  physically left-most bits in a binary mask word.
- *  The unused mask bits are set to zero.
- */
 #define RC_VEC_GETMASKW(maskw, vec) \
     ((maskw) = _mm_movemask_epi8(vec))
 
+#define RC_VEC_SETMASKV(vec, maskv)                                     \
+do {                                                                    \
+    rc_vec_t v_ = (maskv);                                              \
+    /* From {a, b, x, ...}, make {a, a, ..., [7] = a, b, b, ..., b} */  \
+    rc_vec_t vh16l16_ = _mm_unpacklo_epi8(v_, v_);                      \
+    rc_vec_t vh32l32_ = _mm_unpacklo_epi8(vh16l16_, vh16l16_);          \
+    rc_vec_t vh64l64_ = _mm_unpacklo_epi8(vh32l32_, vh32l32_);          \
+    rc_vec_t mask_ = _mm_setr_epi8(1<<0, 1<<1, 1<<2, 1<<3,              \
+                                   1<<4, 1<<5, 1<<6, 1<<7,              \
+                                   1<<0, 1<<1, 1<<2, 1<<3,              \
+                                   1<<4, 1<<5, 1<<6, 1<<7);             \
+    rc_vec_t andv_ = _mm_and_si128(vh64l64_, mask_);                    \
+    (vec) = _mm_cmpeq_epi8(andv_, mask_);                               \
+} while (0)
 
-/*
- * -------------------------------------------------------------
- *  Reductions
- * -------------------------------------------------------------
- */
-
-/**
- *  Count bits in all vector fields.
- *  The format of the accumulator vector is implementation-specific,
- *  but RC_VEC_CNTV() and RC_VEC_CNTR() together computes the bit count.
- *  The accumulation step can be iterated at most RC_VEC_CNTN times
- *  before the reduction step.
- */
 #define RC_VEC_CNTN 1024 /* 8191 untestable */
 
-/**
- *  Count bits in all vector fields, accumulation step.
- */
 #define RC_VEC_CNTV(accv, srcv)                                  \
 do {                                                             \
     rc_vec_t c1__  = _mm_set1_epi8(0x55);                        \
@@ -502,24 +286,11 @@ do {                                                             \
     (accv) = _mm_add_epi32(accv, acc__);                         \
 } while (0)
 
-/**
- *  Count bits in all vector fields, reduction step.
- */
 #define RC_VEC_CNTR(cnt, accv) \
     RC_VEC_SUMR(cnt, accv)
 
-/**
- *  Sum all 8-bit fields.
- *  The format of the accumulator vector is implementation-specific,
- *  but RC_VEC_SUMV() and RC_VEC_SUMR() together computes the sum.
- *  The accumulation step can be iterated at most RC_VEC_SUMN times
- *  before the reduction step.
- */
 #define RC_VEC_SUMN 128
 
-/**
- *  Sum all 8-bit fields, accumulation step.
- */
 #define RC_VEC_SUMV(accv, srcv)                                  \
 do {                                                             \
     rc_vec_t sv__ = (srcv);                                      \
@@ -528,9 +299,6 @@ do {                                                             \
     (accv) = _mm_add_epi16(accv, _mm_unpackhi_epi8(sv__, zv__)); \
 } while (0)
 
-/**
- *  Sum all 8-bit fields, reduction step.
- */
 #define RC_VEC_SUMR(sum, accv)                             \
 do {                                                       \
     rc_vec_t av__ = (accv);                                \
@@ -543,18 +311,8 @@ do {                                                       \
     (sum) = _mm_cvtsi128_si32(rv__);                       \
 } while (0)
 
-/**
- *  Multiply and accumulate all 8-bit fields.
- *  The format of the accumulator vector is implementation-specific,
- *  but RC_VEC_MACV() and RC_VEC_MACR() together computes the sum.
- *  The accumulation step can be iterated at most RC_VEC_MACN times
- *  before the reduction step.
- */
 #define RC_VEC_MACN 1024 /* 16512 untestable */
 
-/**
- *  Multiply and accumulate all 8-bit fields, accumulation step.
- */
 #define RC_VEC_MACV(accv, srcv1, srcv2)              \
 do {                                                 \
     rc_vec_t sv1__ = (srcv1);                        \
@@ -570,9 +328,6 @@ do {                                                 \
     (accv) = _mm_add_epi32(accv, hi1__);             \
 } while (0)
 
-/**
- *  Multiply and accumulate all 8-bit fields, reduction step.
- */
 #define RC_VEC_MACR(mac, accv)                             \
 do {                                                       \
     rc_vec_t av__ = (accv);                                \
@@ -584,7 +339,6 @@ do {                                                       \
     (mac) = _mm_cvtsi128_si32(rv__);                       \
 } while (0)
 
-
 /*
  * -------------------------------------------------------------
  *  Internal support macros
