@@ -1,4 +1,4 @@
-/*  Copyright (C) 2005-2010, Axis Communications AB, LUND, SWEDEN
+/*  Copyright (C) 2005-2016, Axis Communications AB, LUND, SWEDEN
  *
  *  This file is part of RAPP.
  *
@@ -41,47 +41,16 @@
 #include <stdint.h>   /* UINT*_C()       */
 #include "rc_word.h"  /* Word operations */
 
+/* See the porting documentation for generic comments. */
 
-/*
- * -------------------------------------------------------------
- *  Performance hints
- * -------------------------------------------------------------
- */
-
-/**
- *  Use CMPGT instead of CMPGE when possible.
- */
 #define RC_VEC_HINT_CMPGT
 
-/**
- *  Use AVGT/SUBHT instead of AVGR/SUBHR when possible.
- */
 #define RC_VEC_HINT_AVGT
 
-
-/*
- * -------------------------------------------------------------
- *  The vector type
- * -------------------------------------------------------------
- */
-
-/**
- *  The vector type definition.
- */
 typedef rc_word_t rc_vec_t;
 
-/**
- *  The number of bytes in a vector.
- */
 #define RC_VEC_SIZE RC_WORD_SIZE
 
-
-/*
- * -------------------------------------------------------------
- *  Vector state
- * -------------------------------------------------------------
- */
-
 /**
  *  No global declarations is needed for SWAR registers.
  */
@@ -92,35 +61,12 @@ typedef rc_word_t rc_vec_t;
  */
 #define RC_VEC_CLEANUP()
 
-
-/*
- * -------------------------------------------------------------
- *  Memory access
- * -------------------------------------------------------------
- */
-
-/**
- *  Load a vector from memory.
- */
 #define RC_VEC_LOAD(vec, ptr) \
     ((vec) = RC_WORD_LOAD(ptr))
 
-/**
- *  Store a vector in memory.
- */
 #define RC_VEC_STORE(ptr, vec) \
     RC_WORD_STORE(ptr, vec)
 
-
-/*
- * -------------------------------------------------------------
- *  Misaligned memory access
- * -------------------------------------------------------------
- */
-
-/**
- *  Initialize a sequence of misaligned loads.
- */
 #define RC_VEC_LDINIT(vec1, vec2, vec3, uptr, ptr)                           \
 do {                                                                         \
     const uint8_t *sptr__, *uptr__;                                          \
@@ -135,9 +81,6 @@ do {                                                                         \
     (uptr) = uptr__ + RC_VEC_SIZE;            /* Advance data pointer     */ \
 } while (0)
 
-/**
- *  Misaligned vector load from memory.
- */
 #define RC_VEC_LOADU(dstv, vec1, vec2, vec3, uptr) \
 do {                                               \
     rc_vec_t vec__ = RC_WORD_LOAD(uptr);           \
@@ -146,56 +89,24 @@ do {                                               \
     (vec3) = vec__;                                \
 } while (0)
 
-
-/*
- * -------------------------------------------------------------
- *  Field relocation
- * -------------------------------------------------------------
- */
-
-/**
- *  Initialize a field shift.
- */
 #define RC_VEC_SHINIT(shv, bytes) \
     ((shv) = (bytes))
 
-/**
- *  Shift all fields to the logical left.
- */
 #define RC_VEC_SHL(dstv, srcv, shv) \
     ((dstv) = RC_WORD_SHL(srcv, 8*(shv)))
 
-/**
- *  Shift fields to the logical right.
- */
 #define RC_VEC_SHR(dstv, srcv, shv) \
     ((dstv) = RC_WORD_SHR(srcv, 8*(shv)))
 
-/**
- *  Shift all fields to the logical left by a constant value.
- */
 #define RC_VEC_SHLC(dstv, srcv, bytes) \
     RC_VEC_SHL(dstv, srcv, bytes)
 
-/**
- *  Shift fields to the logical right by a constant value.
- */
 #define RC_VEC_SHRC(dstv, srcv, bytes) \
     RC_VEC_SHR(dstv, srcv, bytes)
 
-/**
- *  Align srcv1 and srcv2 to dstv, starting at field @e bytes
- *  into concatenation of srcv1 and srcv2. The alignment
- *  value @e bytes must be a constant.
- */
 #define RC_VEC_ALIGNC(dstv, srcv1, srcv2, bytes) \
     ((dstv) = RC_WORD_ALIGN(srcv1, srcv2, 8*(bytes)))
 
-/**
- *  Pack the even fields of srcv1 and srcv2 into one vector,
- *  with fields 0, ..., RC_VEC_SIZE/2 - 1 from srcv1 and
- *  fields RC_VEC_SIZE/2, ..., RC_VEC_SIZE - 1 from srcv2.
- */
 #if   RC_VEC_SIZE == 2
 #define RC_VEC_PACK(dstv, srcv1, srcv2)        \
 do {                                           \
@@ -238,72 +149,30 @@ do {                                            \
 } while (0)
 #endif
 
-
-/*
- * -------------------------------------------------------------
- *  Bitwise logical operations
- * -------------------------------------------------------------
- */
-
-/**
- *  Set all bits to zero.
- */
 #define RC_VEC_ZERO(vec) \
     ((vec) = 0)
 
-/**
- *  Bitwise NOT.
- */
 #define RC_VEC_NOT(dstv, srcv) \
     ((dstv) = ~(srcv))
 
-/**
- *  Bitwise AND.
- */
 #define RC_VEC_AND(dstv, srcv1, srcv2) \
     ((dstv) = (srcv1) & (srcv2))
 
-/**
- *  Bitwise OR.
- */
 #define RC_VEC_OR(dstv, srcv1, srcv2) \
     ((dstv) = (srcv1) | (srcv2))
 
-/**
- *  Bitwise XOR.
- */
 #define RC_VEC_XOR(dstv, srcv1, srcv2) \
     ((dstv) = (srcv1) ^ (srcv2))
 
-/**
- *  Bitwise AND NOT.
- */
 #define RC_VEC_ANDNOT(dstv, srcv1, srcv2) \
     ((dstv) = (srcv1) & ~(srcv2))
 
-/**
- *  Bitwise OR NOT.
- */
 #define RC_VEC_ORNOT(dstv, srcv1, srcv2) \
     ((dstv) = (srcv1) | ~(srcv2))
 
-/**
- *  Bitwise XOR NOT.
- */
 #define RC_VEC_XORNOT(dstv, srcv1, srcv2) \
     ((dstv) = (srcv1) ^ ~(srcv2))
 
-
-/*
- * -------------------------------------------------------------
- *  Arithmetic operations on 8-bit fields
- * -------------------------------------------------------------
- */
-
-/**
- *  Set a value to all fields.
- *  Sets the scalar value 'scal' in each 8-bit field.
- */
 #define RC_VEC_SPLAT(vec, scal) \
 do {                            \
     (vec)  = (scal) & 0xff;     \
@@ -312,10 +181,6 @@ do {                            \
     RC_VEC_LFOLD__(vec, 32);    \
 } while (0)
 
-/**
- *  Saturating addition.
- *  Computes dstv = MIN(srcv1 + srcv2, 0xff) for each 8-bit field.
- */
 #define RC_VEC_ADDS(dstv, srcv1, srcv2)                               \
 do {                                                                  \
     rc_vec_t sv1__  = (srcv1);                                        \
@@ -329,10 +194,6 @@ do {                                                                  \
     (dstv)  = sum__ | mask__;          /* Saturate                 */ \
 } while (0)
 
-/**
- *  Average value, truncated.
- *  Computes dstv = (srcv1 + srcv2) >> 1 for each 8-bit field.
- */
 #define RC_VEC_AVGT(dstv, srcv1, srcv2)                               \
 do {                                                                  \
     rc_vec_t v1__ = (srcv1);                                          \
@@ -340,10 +201,6 @@ do {                                                                  \
     (dstv) = (v1__ & v2__) + (((v1__ ^ v2__) >> 1) & RC_WORD_C8(7f)); \
 } while (0)
 
-/**
- *  Average value, rounded.
- *  Computes dstv = (srcv1 + srcv2 + 1) >> 1 for each 8-bit field.
- */
 #define RC_VEC_AVGR(dstv, srcv1, srcv2)                                 \
 do {                                                                    \
     rc_vec_t sv1__ = (srcv1);                                           \
@@ -354,11 +211,6 @@ do {                                                                    \
              ( xv__       & RC_WORD_C8(01));  /* Rounding correction */ \
 } while (0)
 
-/**
- *  Average value, rounded towards srcv1.
- *  Computes dstv = (srcv1 + srcv2 + (srcv1 > srcv2)) >> 1
- *  for each 8-bit field.
- */
 #define RC_VEC_AVGZ(dstv, srcv1, srcv2)                                   \
 do {                                                                      \
     rc_vec_t sv1__ = (srcv1);                                             \
@@ -371,10 +223,6 @@ do {                                                                      \
              ( xv__ & (gt__ >> 7) & RC_WORD_C8(01));  /* Rounding corr */ \
 } while (0)
 
-/**
- *  Saturating subtraction.
- *  Computes dstv = MAX(srvc1 - srcv2, 0) for each 8-bit field.
- */
 #define RC_VEC_SUBS(dstv, srcv1, srcv2)                                \
 do {                                                                   \
     rc_vec_t sv1__  = (srcv1);                                         \
@@ -388,10 +236,6 @@ do {                                                                   \
     (dstv)  = diff__ & ~mask__;         /* Saturate                 */ \
 } while (0)
 
-/**
- *  Absolute-value subtraction.
- *  Computes dstv = ABS(srcv1 - srcv2) for each 8-bit field.
- */
 #define RC_VEC_SUBA(dstv, srcv1, srcv2)                                    \
 do {                                                                       \
     rc_vec_t sv1__ = (srcv1);                                              \
@@ -405,25 +249,12 @@ do {                                                                       \
              ((sv2__ & lt__) - (sv1__ & lt__));                            \
 } while (0)
 
-/**
- *  Half subtraction with bias, truncated.
- *  Computes dstv = (srcv1 - srcv2 + 0xff) >> 1 for each 8-bit field.
- */
 #define RC_VEC_SUBHT(dstv, srcv1, srcv2) \
     RC_VEC_AVGT(dstv, srcv1, ~(srcv2))
 
-/**
- *  Half subtraction with bias, rounded.
- *  Computes dstv = (srcv1 - srcv2 + 0x100) >> 1 for each 8-bit field.
- */
 #define RC_VEC_SUBHR(dstv, srcv1, srcv2) \
     RC_VEC_AVGR(dstv, srcv1, ~(srcv2))
 
-/**
- *  Absolute value.
- *  Computes dstv = 2*abs(srcv - 0x80) for each 8-bit field.
- *  The result is saturated to [0,0xff].
- */
 #define RC_VEC_ABS(dstv, srcv)                                          \
 do {                                                                    \
     rc_vec_t sv__   = (srcv);                                           \
@@ -435,17 +266,9 @@ do {                                                                    \
     (dstv) = ((sv__ - mask__) << 1) | mask__; /* Double and saturate */ \
 } while (0)
 
-/**
- *  Comparison.
- *  Computes dstv = srcv1 > srcv2 ? 0x80 : 0 for each 8-bit field.
- */
 #define RC_VEC_CMPGT(dstv, srcv1, srcv2) \
     RC_VEC_AVGT(dstv, srcv1, ~(srcv2))
 
-/**
- *  Comparison.
- *  Computes dst = srcv1 >= srcv2 ? 0x80 : 0 for each 8-bit field.
- */
 #define RC_VEC_CMPGE(dstv, srcv1, srcv2) \
 do {                                     \
     rc_vec_t lt__;                       \
@@ -453,10 +276,6 @@ do {                                     \
     (dstv) = ~lt__;                      \
 } while (0)
 
-/**
- *  Minimum value.
- *  Computes dstv = MIN(srcv1, srcv2) for each 8-bit field.
- */
 #define RC_VEC_MIN(dstv, srcv1, srcv2)                  \
 do {                                                    \
     rc_vec_t sv1__  = (srcv1);                          \
@@ -469,10 +288,6 @@ do {                                                    \
              (sv2__ &  mask__);                         \
 } while (0)
 
-/**
- *  Maximum value.
- *  Computes dstv = MAX(srcv1, srcv2) for each 8-bit field.
- */
 #define RC_VEC_MAX(dstv, srcv1, srcv2)                  \
 do {                                                    \
     rc_vec_t sv1__  = (srcv1);                          \
@@ -485,17 +300,9 @@ do {                                                    \
               (sv2__ & ~mask__);                        \
 } while (0)
 
-/**
- *  Generate the blend vector needed by RC_VEC_LERP().
- */
 #define RC_VEC_BLEND(blendv, blend8) \
     RC_VEC_SPLAT(blendv, blend8)
 
-/**
- *  Linear interpolation.
- *  Computes dstv = srcv1 + ((blend8*(srcv2 - srcv1) + 0x80) >> 8) for each
- *  8-bit field. The Q.8 blend factor @e blend8 must be in the range [0,0x7f].
- */
 #define RC_VEC_LERP(dstv, srcv1, srcv2, blend8, blendv)                      \
 do {                                                                         \
     rc_vec_t sv1__ = (srcv1);                                                \
@@ -510,18 +317,9 @@ do {                                                                         \
     (dstv) = sv1__ + (u1__ | u2__) - (blendv);          /* Merge & unbias  */\
 } while (0)
 
-/**
- *  Generate the blend vector needed by RC_VEC_LERPZ().
- */
 #define RC_VEC_BLENDZ(blendv, blend8) \
     RC_VEC_SPLAT(blendv, blend8)
 
-/**
- *  Linear interpolation rounded towards srcv1.
- *  Computes dstv = srcv1 + (blend8*(srcv2 - srcv1)/256) for each 8-bit
- *  field, with the update term rounded towards zero. The Q.8 blend factor
- *  @e blend8 must be in the range [0,0x7f].
- */
 #define RC_VEC_LERPZ(dstv, srcv1, srcv2, blend8, blendv)                     \
 do {                                                                         \
     rc_vec_t sv1__ = (srcv1);                                                \
@@ -538,18 +336,9 @@ do {                                                                         \
     (dstv) = sv1__ + (u1__ | u2__) - (blendv);         /* Merge & unbias  */ \
 } while (0)
 
-/**
- *  Generate the blend vector needed by RC_VEC_LERPN().
- */
 #define RC_VEC_BLENDN(blendv, blend8) \
     RC_VEC_SPLAT(blendv, blend8)
 
-/**
- *  Linear interpolation rounded towards srcv2.
- *  Computes dstv = srcv1 + (blend8*(srcv2 - srcv1)/256) for each 8-bit
- *  field, with the update term rounded away from zero. The Q.8 blend factor
- *  @e blend8 must be in the range [0,0x7f].
- */
 #define RC_VEC_LERPN(dstv, srcv1, srcv2, blend8, blendv)                     \
 do {                                                                         \
     rc_vec_t sv1__ = (srcv1);                                                \
@@ -566,18 +355,6 @@ do {                                                                         \
     (dstv) = sv1__ + (u1__ | u2__) - (blendv);          /* Merge & unbias */ \
 } while (0)
 
-
-/*
- * -------------------------------------------------------------
- *  Binary mask operations
- * -------------------------------------------------------------
- */
-
-/**
- *  Pack the most significant bits in each 8-bit field to the
- *  physically left-most bits in a binary mask word.
- *  The unused mask bits are set to zero.
- */
 #define RC_VEC_GETMASKW(maskw, vec)                                        \
 do {                                                                       \
     rc_vec_t w__ = ((vec) >> 7) & RC_WORD_C8(01); /* Extract MSBs       */ \
@@ -587,25 +364,23 @@ do {                                                                       \
     (maskw) = w__ & ((1 << RC_VEC_SIZE) - 1);     /* Mask and store     */ \
 } while (0)
 
+/* A definition is useful only when RC_VEC_SIZE is divisible by 8. */
+#if RC_VEC_SIZE == 8
+#define RC_VEC_SETMASKV(vec, maskv)                     \
+do {                                                    \
+    rc_vec_t mw_, v_, av_, nz_, b_;                     \
+    mw_ = RC_WORD_EXTRACT(maskv, 0, 8);                 \
+    RC_VEC_SPLAT(v_, mw_);                              \
+    RC_VEC_AND(av_, v_, RC_WORD_C64(8040201008040201)); \
+    RC_VEC_CMPGT(nz_, av_, 0);                          \
+    nz_ &= RC_WORD_C8(80);                              \
+    b_ = RC_VEC_MASK__(nz_);                            \
+    (vec) = b_;                                         \
+} while (0)
+#endif /* RC_VEC_SIZE == 8 */
 
-/*
- * -------------------------------------------------------------
- *  Reductions
- * -------------------------------------------------------------
- */
-
-/**
- *  Count bits in all vector fields.
- *  The format of the accumulator vector is implementation-specific,
- *  but RC_VEC_CNTV() and RC_VEC_CNTR() together computes the bit count.
- *  The accumulation step can be iterated at most RC_VEC_CNTN times
- *  before the reduction step.
- */
 #define RC_VEC_CNTN 28  /* 31 not divisible by 4 */
 
-/**
- *  Count bits in all vector fields, accumulation step.
- */
 #define RC_VEC_CNTV(accv, srcv)                        \
 do {                                                   \
     rc_vec_t acc__ = (srcv);                           \
@@ -616,9 +391,6 @@ do {                                                   \
     (accv) += (acc__ + (acc__ >> 4)) & RC_WORD_C8(0f); \
 } while (0)
 
-/**
- *  Count bits in all vector fields, reduction step.
- */
 #define RC_VEC_CNTR(cnt, accv) \
 do {                           \
     rc_vec_t sum__ = 0;        \
@@ -626,18 +398,8 @@ do {                           \
     RC_VEC_SUMR(cnt, sum__);   \
 } while (0)
 
-/**
- *  Sum all 8-bit vector fields.
- *  The format of the accumulator vector is implementation-specific,
- *  but RC_VEC_SUMV() and RC_VEC_SUMR() together computes the sum.
- *  The accumulation step can be iterated at most RC_VEC_SUMN times
- *  before the reduction step.
- */
 #define RC_VEC_SUMN 128
 
-/**
- *  Sum all 8-bit vector fields, accumulation step.
- */
 #define RC_VEC_SUMV(accv, srcv)                   \
 do {                                              \
     rc_vec_t sv__ = (srcv);                       \
@@ -645,9 +407,6 @@ do {                                              \
               ((sv__ >> 8) & RC_WORD_C16(00ff));  \
 } while (0)
 
-/**
- *  Sum all 8-bit vector fields, reduction step.
- */
 #if   RC_VEC_SIZE == 2
 #define RC_VEC_SUMR(sum, accv) \
     ((sum) = (accv))
@@ -668,7 +427,6 @@ do {                                                    \
 } while (0)
 #endif
 
-
 /*
  * -------------------------------------------------------------
  *  Internal support macros
@@ -698,7 +456,7 @@ do {                                                    \
 
 /**
  *  Fold a vector to the right.
- *  Computes vec |= vec << bits. The shift amount should be a constant.
+ *  Computes vec |= vec >> bits. The shift amount should be a constant.
  */
 #define RC_VEC_RFOLD__(vec, bits) \
     ((vec) |= ((bits) < 8*RC_VEC_SIZE) ? (vec) >> (bits) : 0)
