@@ -1,4 +1,4 @@
-/*  Copyright (C) 2005-2011, Axis Communications AB, LUND, SWEDEN
+/*  Copyright (C) 2005-2016, Axis Communications AB, LUND, SWEDEN
  *
  *  This file is part of RAPP.
  *
@@ -114,7 +114,9 @@
 typedef struct rc_bmark_data_st {
     uint8_t *dst;     /* Destination buffer large enough for all images */
     uint8_t *src;     /* Source buffer large enough for all images      */
-    uint8_t *aux;     /* Auxilliary buffer large enough for all images  */
+    uint8_t *aux;     /* Auxiliary buffer large enough for all images  */
+    uint8_t *aux2;    /* Second similar auxiliary buffer */
+    uint8_t *map;     /* Buffer for binary mapping.                     */
     int      dim_bin; /* Binary row dimension, with padding             */
     int      dim_u8;  /* 8-bit row dimension, with padding              */
     int      rot_u8;  /* 8-bit rotatated row dimension, no padding      */
@@ -165,9 +167,18 @@ rc_bmark_exec_bin_bin(int (*func)(), const int *args);
 static void
 rc_bmark_exec_bin_bin_m(int (*func)(), const int *args);
 
+static void
+rc_bmark_exec_bin_u8(int (*func)(), const int *args);
+
 static void
 rc_bmark_exec_u8_bin(int (*func)(), const int *args);
 
+static void
+rc_bmark_exec_u8_bin_c(int (*func)(), const int *args);
+
+static void
+rc_bmark_exec_u8_bin_u8_c(int (*func)(), const int *args);
+
 static void
 rc_bmark_exec_u8(int (*func)(), const int *args);
 
@@ -183,6 +194,9 @@ rc_bmark_exec_u8_u8_p(int (*func)(), const int *args);
 static void
 rc_bmark_exec_rotate(int (*func)(), const int *args);
 
+static void
+rc_bmark_exec_thresh_pixel(int (*func)(), const int *args);
+
 
 /*
  * -------------------------------------------------------------
@@ -272,11 +286,16 @@ static const rc_bmark_table_t rc_bmark_suite[] = {
     RC_BMARK_ENTRY(rc_pixop_norm_u8,                      u8_u8,     0, 0),
     /* Type conversions */
     RC_BMARK_ENTRY(rc_type_u8_to_bin,                     u8_bin,    0, 0),
+    RC_BMARK_ENTRY(rc_type_bin_to_u8,                     bin_u8,    0, 0),
     /* Thresholding */
     RC_BMARK_ENTRY(rc_thresh_gt_u8,                       u8_bin,    0, 0),
     RC_BMARK_ENTRY(rc_thresh_lt_u8,                       u8_bin,    0, 0),
     RC_BMARK_ENTRY(rc_thresh_gtlt_u8,                     u8_bin,    0, 0),
     RC_BMARK_ENTRY(rc_thresh_ltgt_u8,                     u8_bin,    0, 0),
+    RC_BMARK_ENTRY(rc_thresh_gt_pixel_u8,                 thresh_pixel, 1, 0),
+    RC_BMARK_ENTRY(rc_thresh_lt_pixel_u8,                 thresh_pixel, 1, 0),
+    RC_BMARK_ENTRY(rc_thresh_gtlt_pixel_u8,               thresh_pixel, 2, 0),
+    RC_BMARK_ENTRY(rc_thresh_ltgt_pixel_u8,               thresh_pixel, 2, 0),
     /* Statistics */
     RC_BMARK_ENTRY(rc_stat_sum_bin,                       bin,       0, 0),
     RC_BMARK_ENTRY(rc_stat_sum_u8,                        u8,        0, 0),
@@ -461,7 +480,13 @@ static const rc_bmark_table_t rc_bmark_suite[] = {
     RC_BMARK_ENTRY(rc_morph_hmt_golay_e_3x3_c8_r315_bin,  bin_bin,   0, 0),
     /* Binary logical margins */
     RC_BMARK_ENTRY(rc_margin_horz_bin,                    p_bin,     0, 0),
-    RC_BMARK_ENTRY(rc_margin_vert_bin,                    p_bin,     0, 0)
+    RC_BMARK_ENTRY(rc_margin_vert_bin,                    p_bin,        0, 0),
+    /* Conditional operations */
+    RC_BMARK_ENTRY(rc_cond_set_u8,                        u8_bin_c,     1, 0),
+    RC_BMARK_ENTRY(rc_cond_addc_u8,                       u8_bin_c,     1, 0),
+    RC_BMARK_ENTRY(rc_cond_subc_u8,                       u8_bin_c,     1, 0),
+    RC_BMARK_ENTRY(rc_cond_copy_u8,                       u8_bin_u8_c,  0, 0),
+    RC_BMARK_ENTRY(rc_cond_add_u8,                        u8_bin_u8_c,  0, 0)
 };
 
 
@@ -675,6 +700,8 @@ rc_bmark_setup(void *lib, int width, int height)
     rc_bmark_data.dst     = (*alloc)(size);
     rc_bmark_data.src     = (*alloc)(size);
     rc_bmark_data.aux     = (*alloc)(size);
+    rc_bmark_data.aux2    = (*alloc)(size);
+    rc_bmark_data.map     = (*alloc)(size);
     rc_bmark_data.dim_bin = dim_bin;
     rc_bmark_data.dim_u8  = dim_u8;
     rc_bmark_data.rot_u8  = rot_u8;
@@ -687,10 +714,17 @@ rc_bmark_setup(void *lib, int width, int height)
     memset(rc_bmark_data.dst, 0, size);
     memset(rc_bmark_data.src, 0, size);
     memset(rc_bmark_data.aux, 0, size);
+    memset(rc_bmark_data.aux2, 0, size);
+
+    memset(&rc_bmark_data.map[0], 0, size/3);
+    memset(&rc_bmark_data.map[dim_bin * (height / 3)], 0xff, size/3);
+    memset(&rc_bmark_data.map[dim_bin * (2 * height / 3)], 0x55, size/3);
 
     rc_bmark_data.dst += offset;
     rc_bmark_data.src += offset;
     rc_bmark_data.aux += offset;
+    rc_bmark_data.aux2 += offset;
+    rc_bmark_data.map += offset;
 }
 
 static void
@@ -699,6 +733,8 @@ rc_bmark_cleanup(void)
     (*rc_bmark_data.release)(&rc_bmark_data.src[-rc_bmark_data.offset]);
     (*rc_bmark_data.release)(&rc_bmark_data.dst[-rc_bmark_data.offset]);
     (*rc_bmark_data.release)(&rc_bmark_data.aux[-rc_bmark_data.offset]);
+    (*rc_bmark_data.release)(&rc_bmark_data.aux2[-rc_bmark_data.offset]);
+    (*rc_bmark_data.release)(&rc_bmark_data.map[-rc_bmark_data.offset]);
 }
 
 static void
@@ -772,6 +808,25 @@ rc_bmark_exec_bin_bin_m(int (*func)(), const int *args)
             args[0]);
 }
 
+static void
+rc_bmark_exec_bin_u8(int (*func)(), const int *args)
+{
+    (void)args;
+    (*func)(rc_bmark_data.dst,   rc_bmark_data.dim_u8,
+            rc_bmark_data.src,   rc_bmark_data.dim_bin,
+            rc_bmark_data.width, rc_bmark_data.height);
+}
+
+static void
+rc_bmark_exec_u8_bin_c(int (*func)(), const int *args)
+{
+    (void)args;
+    (*func)(rc_bmark_data.dst,   rc_bmark_data.dim_u8,
+            rc_bmark_data.map,   rc_bmark_data.dim_bin,
+            rc_bmark_data.width, rc_bmark_data.height,
+            (int)args[0]);
+}
+
 static void
 rc_bmark_exec_u8_bin(int (*func)(), const int *args)
 {
@@ -789,6 +844,16 @@ rc_bmark_exec_u8(int (*func)(), const int *args)
             (int)args[0], (int)args[1]);
 }
 
+static void
+rc_bmark_exec_u8_bin_u8_c(int (*func)(), const int *args)
+{
+    (void)args;
+    (*func)(rc_bmark_data.dst,   rc_bmark_data.dim_u8,
+            rc_bmark_data.src,   rc_bmark_data.dim_u8,
+            rc_bmark_data.map,   rc_bmark_data.dim_bin,
+            rc_bmark_data.width, rc_bmark_data.height);
+}
+
 static void
 rc_bmark_exec_u8_p(int (*func)(), const int *args)
 {
@@ -825,3 +890,22 @@ rc_bmark_exec_rotate(int (*func)(), const int *args)
             rc_bmark_data.src, rc_bmark_data.dim_u8,
             rc_bmark_data.width, rc_bmark_data.height);
 }
+
+static void
+rc_bmark_exec_thresh_pixel(int (*func)(), const int *args)
+{
+    const int num_thresholds = args[0];
+    if (num_thresholds == 2) {
+        (*func)(rc_bmark_data.dst,   rc_bmark_data.dim_bin,
+                rc_bmark_data.src,   rc_bmark_data.dim_u8,
+                rc_bmark_data.aux,   rc_bmark_data.dim_u8,
+                rc_bmark_data.aux2,  rc_bmark_data.dim_u8,
+                rc_bmark_data.width, rc_bmark_data.height);
+    }
+    else {
+        (*func)(rc_bmark_data.dst,   rc_bmark_data.dim_bin,
+                rc_bmark_data.src,   rc_bmark_data.dim_u8,
+                rc_bmark_data.aux,   rc_bmark_data.dim_u8,
+                rc_bmark_data.width, rc_bmark_data.height);
+    }
+}
